; robotfindskitten routines
;
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.

; ===== Routines ===============================================================
calcCol:
	ld	h, a
	ld	e, 6
	call	multhe
	;dec	hl
	ld	a, l
	ld	(penCol), a
	ret

calcRow:
	ld	h, a
	ld	e, 7
	call	multhe
	ld	a, l
	ld	(penRow), a
	ret

calcLoc:
; Location in memory
	push	af
	ld	a, (currentY)
	ld	e, a
	ld	h, 15
	call	multhe
	ld	d, 0
	ld	a, (currentX)
	ld	e, a
	add	hl, de
	ld	de, playField
	add	hl, de
	pop	af
	ret

adjustForNextPage:
	bit	7, h
	ret	z
	inc	a
	ld	de, 0C000h
	add	hl, de
	ret

wait:
	ld	a, ' '
	push	hl
	push	de
	ld	hl, 400h
	ld	de, 0
waitLoop:
	dec	hl	; 10
	b_call(_PutMap)
	call	cphlde
	jr	nz, waitLoop	; 12
	pop	de
	pop	hl

;ldHLindirect:
;	ld	a, (hl)
;	inc	hl
;	ld	h, (hl)
;	ld	l, a	
;	ret

cphlde:
	or	a
	sbc	hl,de
	add	hl,de
	ret

VPutS:
; Optimized from the SDK.  Needed because I need it to return the NC flag intact.
	ld	a, (hl)
	inc	hl
	or	a
	ret	z
	cp	' '
	jr	nz, VPutDoOnce
	push	af
	b_call(_VPutMap) ; Smooths out the scrolling.
	pop	af
VPutDoOnce:
	b_call(_VPutMap)
	jr	nc, VPutS ; carry on fail
	ret


; ----- Milos Math Routines ----------------------------------------------------
; These routines are by Milos "baze" Bazelides, from the web page
; http://baze.au.com/misc/z80bits.html

multhe:
; Input: H = Multiplier, E = Multiplicand, L = 0, D = 0
; Output: HL = Product 
	xor	a
	ld	l, a
	ld	d, a
;	ld	l, 0
;	ld	d, 0
	sla	h		; optimised 1st iteration
	jr	nc,$+3		;
	ld	l,e		;
	ld	b, 7
multheLoop:	
	add	hl,hl		; unroll 7 times
	jr	nc,$+3		; ...
	add	hl,de		; ...
	djnz	multheLoop
	ret

; ----- LFSR -------------------------------------------------------------------
;James Montelongo
;optimized by Spencer Putt
;out:
; a = 8 bit random number
RandLFSR:
        ld hl,LFSRSeed+4
        ld e,(hl)
        inc hl
        ld d,(hl)
        inc hl
        ld c,(hl)
        inc hl
        ld a,(hl)
        ld b,a
        rl e \ rl d
        rl c \ rla
        rl e \ rl d
        rl c \ rla
        rl e \ rl d
        rl c \ rla
        ld h,a
        rl e \ rl d
        rl c \ rla
        xor b
        rl e \ rl d
        xor h
        xor c
        xor d
        ld hl,LFSRSeed+6
        ld de,LFSRSeed+7
        ld bc,7
        lddr
        ld (de),a
        ret
